function [EnergyProfileCell] = energy_profile_islands(powerLog,RuntimeCell,N)
%energy_profile_islands
%   energy_profile_islands(powerLog,RuntimeCell,N) adequates the energy 
%   measurement to the runtime of the algorithm. 
%   @param powerLog: energy log 
%   @param RuntimeCell: time log for all workers (Cell object)
%   @param N: sample right before the energy actually starts rising its value 
%	over the idle state.

%% Initialize variables 
RUNTIME_LENGTH = cellfun(@length, RuntimeCell);
WORKERS = length(RUNTIME_LENGTH);
COMM = RuntimeCell{1,1}(1,3);
POP = RuntimeCell{1,1}(1,4);
GEN = RuntimeCell{1,1}(1,5);

%% powerGround assignment
% In this version 'N' equals '0' due to measurement perfectly synchronised
% with MATLAB algorithms' execution.
if nargin < 3
    powerGround = 65.38; % mean measured 30 June 2017 (1.5hours)
    N = 1;
else
    powerGround = mean(powerLog(1:N,1));
end

%% powerLog preparation
% Is assumed to be reading the compute-0-1 columns (3rd and 7th)
powerLog = powerLog(:,[3,7]);

%% Runtime preparation for all Workers

for w = 1:WORKERS        
    %% diff_runtime cell-matrix construction:
    % First row is header where the algorithm's initial conditions are set.
    %   (:,1) is for deltaTime
    %   (:,2) is for deltaEnergy
    %   (:,3) is for #thread
    %   (:,4) is for #communication
    %   (:,5) is for population
    %   (:,6) is for #generation
    %   (:,7) is for step in the runtime
    runtime = []; diffRuntime = []; % Clear variables for loop
    runtime = RuntimeCell{w,1};
    diffRuntime(:,1) = diff(runtime(:,1));
    diffRuntime(:,3:7) = runtime(2:end,2:6); 

    %% Energy estimation
    realTime = []; En = []; energy = []; % clear for loop in WORKERS
    En0=powerLog(N,2);
    powerLogLength = length(powerLog) - 1; % Last measurement is reserved
    timeElapsed = runtime(end,1);
    delay = (timeElapsed + 1) - powerLogLength;
    for (i=1:RUNTIME_LENGTH(w)-1)
        % A correction is needed due to imperfect measurement of energy,
	% it means, f ~= 1Hz, actually, f < 1Hz.
        realTime(i) = N + runtime(i+1,1) - delay*runtime(i+1)/timeElapsed;
        if (i==1)
            En(i)=( (powerLog(floor(realTime(i)) + 1, 2) - ...
                     powerLog(floor(realTime(i)), 2)) * ...
                     (realTime(i) - floor(realTime(i))) ) + ... 
                     powerLog(floor(realTime(i)), 2);
            energy=En(i)-En0;
        else
            En(i)=( (powerLog(floor(realTime(i))+1,2) - ...
                powerLog(floor(realTime(i)),2)) * ...
                (realTime(i) - floor(realTime(i))) ) + ...
                powerLog(floor(realTime(i)),2);
            energy=En(i)-En(i-1);
        end  
        diffRuntime(i,2)=energy-(powerGround*diffRuntime(i,1)/3600);
    end 
    
    %% Create Cell with all workers
    EnergyProfileCell{w,1} = diffRuntime;
end
end
